<?php
defined( 'ABSPATH' ) || die();

class WPCP_HTML {

	/**
	 * Text input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function text_input( $field ) {
		global $thepostid, $post;
		$thepostid  = empty( $thepostid ) ? $post->ID : $thepostid;
		$args       = apply_filters(
			'wpcp_text_input',
			wp_parse_args(
				$field,
				array(
					'type'          => 'text',
					'label'         => '',
					'name'          => '',
					'id'            => '',
					'value'         => '',
					'default'       => '',
					'size'          => 'regular',
					'class'         => 'short',
					'wrapper_class' => '',
					'placeholder'   => '',
					'tooltip'       => '',
					'desc'          => '',
					'css'           => '',
					'after'         => '',
					'attrs'         => array(),
				)
			)
		);
		$args['id'] = ! empty( $args['id'] ) ? $args['id'] : $args['name'];
		if ( empty( $args['value'] ) && get_post_meta( $thepostid, $field['name'], true ) ) {
			$args['value'] = (string) get_post_meta( $thepostid, $field['name'], true );
		} else {
			$args['value'] = $args['default'];
		}
		if ( ! empty( $args['tooltip'] ) ) {
			$args['tooltip'] = sprintf( '<span class="wpcp-tooltip" data-tip="%1$s">[?]</span>', wp_kses_post( $field['tooltip'] ) );
		}
		$attributes = self::implode_html_attributes(
			$args['attrs'],
			array(
				'placeholder' => $args['placeholder'],
				'style'       => $args['css'],
			)
		);
		?>
		<p class="form-field wpcp-field <?php echo sanitize_html_class( $args['id'] ); ?>-field <?php echo sanitize_html_class( $args['type'] ); ?>-input <?php echo sanitize_html_class( $args['wrapper_class'] ); ?>">
			<?php if ( ! empty( $args['label'] ) ) : ?>
				<label for="<?php echo esc_attr( $args['id'] ); ?>" class="wpcp-label"><?php echo esc_html( $args['label'] ); ?><?php echo wp_kses_post( $args['tooltip'] ); ?></label>
			<?php endif; ?>
			<input type="<?php echo esc_attr( $args['type'] ); ?>" class="wpcp-input <?php echo esc_attr( $args['size'] ); ?>-text <?php echo esc_attr( $args['class'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" value="<?php echo esc_attr( $args['value'] ); ?>" autocomplete="off" <?php echo wp_kses_post( $attributes ); ?>/>

			<?php if ( ! empty( $args['after'] ) ) : ?>
				<?php echo wp_kses_post( $args['after'] ); ?>
			<?php endif; ?>

			<?php if ( ! empty( $args['desc'] ) ) : ?>
				<span class="wpcp-desc description"><?php echo wp_kses_post( $args['desc'] ); ?></span>
			<?php endif; ?>
		</p>
		<?php
	}


	/**
	 * Textarea input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function textarea_input( $field ) {
		global $thepostid, $post;
		$thepostid = empty( $thepostid ) ? $post->ID : $thepostid;
		$args      = apply_filters(
			'wpcp_textarea_input',
			wp_parse_args(
				$field,
				array(
					'type'          => 'text',
					'label'         => '',
					'name'          => '',
					'id'            => '',
					'value'         => '',
					'default'       => '',
					'size'          => 'regular',
					'class'         => 'short',
					'wrapper_class' => '',
					'placeholder'   => '',
					'tooltip'       => '',
					'desc'          => '',
					'css'           => '',
					'rows'          => '2',
					'cols'          => '20',
					'attrs'         => array(),
				)
			)
		);

		$args['id'] = ! empty( $args['id'] ) ? $args['id'] : $args['name'];
		if ( empty( $args['value'] ) && ! empty( $thepostid ) && get_post_meta( $thepostid, $args['name'], true ) ) {
			$args['value'] = (string) get_post_meta( $thepostid, $args['name'], true );
		} else {
			$args['value'] = $args['default'];
		}
		$attributes = self::implode_html_attributes(
			$args['attrs'],
			array(
				'placeholder' => $args['placeholder'],
				'style'       => $args['css'],
				'rows'        => $args['rows'],
				'cols'        => $args['cols'],
			)
		);

		if ( ! empty( $args['tooltip'] ) ) {
			$args['tooltip'] = sprintf( '<span class="wpcp-tooltip" data-tip="%1$s">[?]</span>', wp_kses_post( $args['tooltip'] ) );
		}
		?>
		<p class="form-field wpcp-field <?php echo sanitize_html_class( $args['id'] ); ?>-field <?php echo sanitize_html_class( $args['type'] ); ?>-input <?php echo sanitize_html_class( $args['wrapper_class'] ); ?>">
			<?php if ( ! empty( $args['label'] ) ) : ?>
				<label for="<?php echo esc_attr( $args['id'] ); ?>" class="wpcp-label"><?php echo esc_html( $args['label'] ); ?><?php echo wp_kses_post( $args['tooltip'] ); ?></label>
			<?php endif; ?>
			<textarea class="wpcp-input <?php echo esc_attr( $args['size'] ); ?>-text <?php echo esc_attr( $args['class'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" autocomplete="off" <?php echo wp_kses_post( $attributes ); ?>><?php echo esc_textarea( $args['value'] ); ?></textarea>

			<?php if ( ! empty( $args['desc'] ) ) : ?>
				<span class="wpcp-desc description"><?php echo wp_kses_post( $args['desc'] ); ?></span>
			<?php endif; ?>
		</p>
		<?php
	}

	/**
	 * Checkbox input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function checkbox_input( $field ) {
		global $thepostid, $post;
		$thepostid = empty( $thepostid ) ? $post->ID : $thepostid;
		$args      = apply_filters(
			'wpcp_checkbox_input',
			wp_parse_args(
				$field,
				array(
					'type'          => 'text',
					'label'         => '',
					'name'          => '',
					'id'            => '',
					'value'         => '',
					'default'       => '',
					'size'          => 'regular',
					'class'         => 'short',
					'wrapper_class' => '',
					'placeholder'   => '',
					'tooltip'       => '',
					'desc'          => '',
					'css'           => '',
					'cbvalue'       => 'on',
					'attrs'         => array(),
				)
			)
		);

		$args['id'] = esc_attr( ! empty( $args['id'] ) ? $args['id'] : $args['name'] );
		if ( empty( $args['value'] ) && ! empty( $thepostid ) && get_post_meta( $thepostid, $args['name'], true ) ) {
			$args['value'] = (string) get_post_meta( $thepostid, $args['name'], true );
		} else {
			$args['value'] = $args['default'];
		}
		$attributes = self::implode_html_attributes(
			$args['attrs'],
			array(
				'style' => $args['css'],
			)
		);

		if ( ! empty( $args['tooltip'] ) ) {
			$args['tooltip'] = sprintf( '<span class="wpcp-tooltip" data-tip="%1$s">[?]</span>', wp_kses_post( $args['tooltip'] ) );
		}
		?>
			<p class="form-field wpcp-field <?php echo sanitize_html_class( $args['id'] ); ?>-field <?php echo sanitize_html_class( $args['type'] ); ?>-input <?php echo sanitize_html_class( $args['wrapper_class'] ); ?>">
				<label for="<?php echo esc_attr( $args['id'] ); ?>" class="wpcp-label">
				<input type="checkbox" class="<?php echo esc_attr( $args['class'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" value="<?php echo esc_attr( $args['cbvalue'] ); ?>" <?php checked( $args['value'], $args['cbvalue'] ); ?> <?php echo wp_kses_post( $attributes ); ?>/>
					<?php echo wp_kses_post( $args['label'] ); ?><?php echo wp_kses_post( $args['tooltip'] ); ?>
				</label>
				<?php echo ! empty( $args['desc'] ) ? sprintf( '<span class="wpcp-desc description">%s</span>', wp_kses_post( $args['desc'] ) ) : ''; ?>
			</p>
		<?php
	}

	/**
	 * Render select input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function select_input( $field ) {
		global $thepostid, $post;
		$thepostid = empty( $thepostid ) ? $post->ID : $thepostid;

		$args = apply_filters(
			'wpcp_select_input',
			wp_parse_args(
				$field,
				array(
					'label'         => '',
					'name'          => null,
					'options'       => array(),
					'value'         => array(),
					'default'       => array(),
					'class'         => '',
					'wrapper_class' => '',
					'id'            => '',
					'tooltip'       => '',
					'css'           => '',
					'size'          => 'large',
					'placeholder'   => __( '-- Please Select --', 'wp-content-pilot' ),
					'multiple'      => false,
					'attrs'         => array(),
				)
			)
		);

		$args['id'] = esc_attr( ! empty( $args['id'] ) ? $args['id'] : $args['name'] );
		if ( empty( $args['value'] ) && ! empty( $thepostid ) && get_post_meta( $thepostid, sanitize_key( $field['name'] ), true ) ) {
			$args['value'] = get_post_meta( $thepostid, sanitize_key( $field['name'] ), true );
		} else {
			$args['value'] = $args['default'];
		}

		$attributes = self::implode_html_attributes(
			$args['attrs'],
			array(
				'style' => $args['css'],
			)
		);

		if ( ! empty( $args['tooltip'] ) ) {
			$args['tooltip'] = sprintf( '<span class="wpcp-tooltip" data-tip="%1$s">[?]</span>', wp_kses_post( $field['tooltip'] ) );
		}
		?>
		<p class="form-field wpcp-field <?php echo sanitize_html_class( $args['id'] ); ?>-field select-input <?php echo sanitize_html_class( $args['wrapper_class'] ); ?>">
			<?php if ( ! empty( $args['label'] ) ) : ?>
				<label for="<?php echo esc_attr( $args['id'] ); ?>" class="wpcp-label"><?php echo esc_html( $args['label'] ); ?><?php echo wp_kses_post( $args['tooltip'] ); ?></label>
			<?php endif; ?>
			<select class="wpcp-input <?php echo esc_attr( $args['size'] ); ?>-text <?php echo esc_attr( $args['class'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" <?php echo wp_kses_post( $attributes ); ?> <?php echo $args['multiple'] ? 'multiple' : ''; ?>>
				<?php foreach ( $args['options'] as $key => $value ) : ?>
					<option value="<?php echo esc_attr( $key ); ?>" <?php self::is_selected( $key, $args['value'] ); ?>><?php echo esc_html( $value ); ?></option>
				<?php endforeach; ?>
			</select>

			<?php if ( ! empty( $args['desc'] ) ) : ?>
				<span class="wpcp-desc description"><?php echo wp_kses_post( $args['desc'] ); ?></span>
			<?php endif; ?>
		</p>
		<?php
	}


	/**
	 * Render range input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function range_input( $field ) {
		$field = wp_parse_args(
			$field,
			array(
				'class' => '',
				'attrs' => [],
				'min'   => 0,
				'max'   => 500,
				'skin'  => 'round',
			)
		);

		$field['class'] .= ' wpcp-range-slider ';
		$field['attrs']  = wp_parse_args(
			$field['attrs'],
			array(
				'data-min'  => $field['min'],
				'data-max'  => $field['max'],
				'data-skin' => $field['skin'],
			)
		);

		self::text_input( $field );
	}

	/**
	 * Render switch input field
	 *
	 * @param array $field Field arguments.
	 *
	 * @return void
	 */
	public static function switch_input( $field ) {
		global $thepostid, $post;
		$thepostid = empty( $thepostid ) ? $post->ID : $thepostid;

		$args = apply_filters(
			'wpcp_switch_input',
			wp_parse_args(
				$field,
				array(
					'label'         => false,
					'name'          => '',
					'id'            => '',
					'value'         => '',
					'default'       => '',
					'check'         => 'on',
					'css'           => '',
					'class'         => '',
					'wrapper_class' => '',
					'tooltip'       => '',
					'desc'          => '',
					'attrs'         => [],
				)
			)
		);

		$args['id'] = esc_attr( ! empty( $args['id'] ) ? $args['id'] : $args['name'] );
		if ( empty( $args['value'] ) && ! empty( $thepostid ) && get_post_meta( $thepostid, $field['name'], true ) ) {
			$args['value'] = (string) get_post_meta( $thepostid, $field['name'], true );
		} else {
			$args['value'] = $args['default'];
		}
		$attributes = self::implode_html_attributes(
			$args['attrs'],
			array(
				'style' => $args['css'],
			)
		);

		if ( ! empty( $args['tooltip'] ) ) {
			$args['tooltip'] = sprintf( '<span class="wpcp-tooltip" data-tip="%1$s">[?]</span>', wp_kses_post( $args['tooltip'] ) );
		}
		?>
		<p class="form-field wpcp-field  wpcp-switch <?php echo sanitize_html_class( $args['id'] ); ?>-field switch-input <?php echo sanitize_html_class( $args['wrapper_class'] ); ?>">
			<label for="<?php echo esc_attr( $args['id'] ); ?>" class="wpcp-label">
				<input type="checkbox" class="<?php echo esc_attr( $args['class'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" value="<?php echo esc_attr( $args['check'] ); ?>" <?php checked( $args['value'], $args['check'] ); ?> <?php echo wp_kses_post( $attributes ); ?>/>
				<span class="wpcp-switch"></span>
				<?php echo esc_html( $args['label'] ); ?><?php echo wp_kses_post( $args['tooltip'] ); ?>
			</label>
			<?php echo ! empty( $args['desc'] ) ? sprintf( '<span class="wpcp-desc description">%s</span>', wp_kses_post( $args['desc'] ) ) : ''; ?>
		</p>
		<?php
	}


	/**
	 * Render description for input field.
	 *
	 * @param array $field Field arguments.
	 *
	 * @return string
	 */
	public static function input_desc( $field ) {
		$field = wp_parse_args(
			$field,
			array(
				'id'   => '',
				'desc' => '',
			)
		);

		return sprintf(
			'<span class="wpcp-input-desc">%s</span>',
			wp_kses(
				$field['desc'],
				array(
					'a'      => array(
						'class' => true,
						'href'  => true,
					),
					'button' => array( 'class' => true ),
					'i'      => array( 'class' => true ),
				)
			)
		);
	}


	/**
	 * @param $value
	 * @param $options
	 *
	 * @return string
	 */
	public static function is_selected( $value, $options ) {

		if ( is_array( $options ) ) {
			$options = array_map( 'intval', $options );

			return selected( in_array( $value, $options, true ), true, false );
		}

		return selected( $value, $options, false );
	}


	/**
	 * @param       $raw_attributes
	 * @param array $extra
	 *
	 * @return string
	 */
	public static function implode_html_attributes( $raw_attributes, $extra = array() ) {
		$attributes = array();
		foreach ( array_merge( $raw_attributes, $extra ) as $name => $value ) {
			$attributes[] = esc_attr( $name ) . '="' . esc_attr( $value ) . '"';
		}

		return implode( ' ', $attributes );
	}

	/**
	 * @param string $class
	 *
	 * @since 1.2.0
	 * @return string
	 */
	public static function start_double_columns( $class = '' ) {
		return sprintf( '<div class="wpcp-dc %s">', sanitize_html_class( $class ) );
	}

	/**
	 * @since 1.2.0
	 * @return string
	 */
	public static function end_double_columns() {
		return '</div><!--wpcp-dc-->';
	}

}
