<?php

defined( 'ABSPATH' ) || exit();

class WPCP_Logger {
	/**
	 * $file - file
	 *
	 * @var resource
	 */
	protected $file;

	/**
	 * $options - settable options - future use - passed through constructor
	 *
	 * @var array
	 */
	protected $options = array(
		'dateFormat' => 'd-M-Y H:i:s',
	);

	/**
	 * @var WPCP_Logger
	 */
	protected static $instance = null;

	/**
	 * @var bool
	 */
	protected $debug = false;

	/**
	 * Constructor - open log file.
	 *
	 * @param string $log_file - path and log file name.
	 * @param array  $params - settable options - future use - passed through constructor.
	 *
	 * @since 1.2.0
	 * WPCP_Logger constructor.
	 */
	public function __construct( $log_file = 'debug.log', $params = array() ) {}

	/**
	 * Debug method (write debug message)
	 *
	 * @param string $message - message to write to log file.
	 * @param int    $campaign_id - campaign id.
	 *
	 * @return void
	 */
	public function debug( $message, $campaign_id = 0 ) {
		$this->write_log( $message, 'DEBUG', $campaign_id );
	}

	/**
	 * Info method (write info message)
	 *
	 * @param string $message - message to write to log file.
	 * @param int    $campaign_id - campaign id.
	 *
	 * @return void
	 */
	public function info( $message, $campaign_id = 0 ) {
		$this->write_log( $message, 'INFO', $campaign_id );
	}

	/**
	 * Warning method (write warning message)
	 *
	 * @param string $message - message to write to log file.
	 * @param int    $campaign_id - campaign id.
	 *
	 * @return void
	 */
	public function warning( $message, $campaign_id = 0 ) {
		$this->write_log( $message, 'WARNING', $campaign_id );
	}

	/**
	 * Error method (write error message)
	 *
	 * @param string $message - message to write to log file.
	 * @param int    $campaign_id - campaign id.
	 *
	 * @return void
	 */
	public function error( $message, $campaign_id = 0 ) {
		$this->write_log( $message, 'ERROR', $campaign_id = 0 );
	}

	/**
	 * Write to log file
	 *
	 * @param string $message - message to write to log file.
	 * @param string $severity - severity of message.
	 * @param int    $campaign_id - campaign id.
	 *
	 * @return void
	 */
	public function write_log( $message, $severity, $campaign_id = 0 ) {
		$severities = [ 'INFO', 'WARNING', 'ERROR' ];
		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			$severities[] = 'DEBUG';
		}

		if ( in_array( $severity, $severities, true ) ) {
			wpcp_insert_log( $message, $severity, $campaign_id );
		}

	}

	/**
	 * @return WPCP_Logger
	 * @since 1.2.0
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

/**
 * @return WPCP_Logger
 * @since 1.2.0
 */
function wpcp_logger() {
	return WPCP_Logger::instance();
}

wpcp_logger();
